# coding=utf-8

import logging
import sys

try:
    import psycopg2.extensions
except ImportError as e:
    from django.core.exceptions import ImproperlyConfigured
    raise ImproperlyConfigured("Error loading psycopg2 module: %s" % e)

try:
    from gevent.lock import Semaphore
except ImportError:
    from eventlet.semaphore import Semaphore

from django.db.backends.postgresql.base import DatabaseWrapper as OriginalDatabaseWrapper

from . import creation, psycopg2_pool

logger = logging.getLogger('django.geventpool')

connection_pools = {}
connection_pools_lock = Semaphore(value=1)


class DatabaseWrapperMixin(object):
    def __init__(self, *args, **kwargs):
        self._pool = None
        super(DatabaseWrapperMixin, self).__init__(*args, **kwargs)
        self.creation = creation.DatabaseCreation(self)

    @property
    def pool(self):
        if self._pool is not None:
            return self._pool
        connection_pools_lock.acquire()
        if self.alias not in connection_pools:
            self._pool = psycopg2_pool.PostgresConnectionPool(
                **self.get_connection_params())
            connection_pools[self.alias] = self._pool
        else:
            self._pool = connection_pools[self.alias]
        connection_pools_lock.release()
        return self._pool

    def get_new_connection(self, conn_params):
        if self.connection is None:
            self.connection = self.pool.get()
            self.closed_in_transaction = False
        return self.connection

    def get_connection_params(self):
        conn_params = super(DatabaseWrapperMixin, self).get_connection_params()
        for attr in ['MAX_CONNS', 'REUSE_CONNS']:
            if attr in self.settings_dict['OPTIONS']:
                conn_params[attr] = self.settings_dict['OPTIONS'][attr]
        return conn_params

    def close(self):
        self.validate_thread_sharing()
        if self.closed_in_transaction or self.connection is None:
            return  # no need to close anything
        try:
            self._close()
        except:
            # In some cases (database restart, network connection lost etc...)
            # the connection to the database is lost without giving Django a
            # notification. If we don't set self.connection to None, the error
            # will occur at every request.
            self.connection = None
            logger.warning(
                'psycopg2 error while closing the connection.',
                exc_info=sys.exc_info())
            raise
        finally:
            self.set_clean()

    def close_if_unusable_or_obsolete(self):
        # Always close the connection because it's not (usually) really being closed.
        self.close()

    def _close(self):
        if self.connection.closed:
            self.pool.closeall()
        else:
            if self.connection.get_transaction_status() == psycopg2.extensions.TRANSACTION_STATUS_INTRANS:
                self.connection.rollback()
                self.connection.autocommit = True
            with self.wrap_database_errors:
                self.pool.put(self.connection)
        self.connection = None

    def closeall(self):
        for pool in connection_pools.values():
            pool.closeall()

    def set_clean(self):
        if self.in_atomic_block:
            self.closed_in_transaction = True
            self.needs_rollback = True


class DatabaseWrapper(DatabaseWrapperMixin, OriginalDatabaseWrapper):
    pass



# coding=utf-8

from functools import wraps
from contextlib import contextmanager

from django.core.signals import request_finished


def close_connection(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        finally:
            request_finished.send(sender='greenlet')
    return wrapper


@contextmanager
def nullcontext(enter_result=None):
    yield enter_result



# coding=utf-8

from django.contrib.gis.db.backends.postgis.base import DatabaseWrapper as OriginalDatabaseWrapper

from django_db_geventpool.backends.postgresql_psycopg2.base import DatabaseWrapperMixin


class DatabaseWrapper(DatabaseWrapperMixin, OriginalDatabaseWrapper):
    pass



# coding=utf-8

from django.db.backends.postgresql.creation import DatabaseCreation as OriginalDatabaseCreation


class DatabaseCreationMixin(object):
    def _create_test_db(self, verbosity, autoclobber, keepdb=False):
        self.connection.closeall()
        return super(DatabaseCreationMixin, self)._create_test_db(verbosity, autoclobber, keepdb)

    def _destroy_test_db(self, test_database_name, verbosity):
        self.connection.closeall()
        return super(DatabaseCreationMixin, self)._destroy_test_db(test_database_name, verbosity)


class DatabaseCreation(DatabaseCreationMixin, OriginalDatabaseCreation):
    pass



# coding=utf-8
import codecs

from setuptools import setup, find_packages


def long_description():
    try:
        with codecs.open('README.rst', 'r', 'utf-8') as f:
            return f.read()
    except:
        return 'Error loading README.rst'


setup(
    name='django-db-geventpool',
    version='4.0.1',
    install_requires=[
        'django>=3.1',
        'psycogreen>=1.0',
    ],
    url='https://github.com/jneight/django-db-geventpool',
    description='Add a DB connection pool using gevent to django',
    long_description=long_description(),
    long_description_content_type='text/x-rst',
    packages=find_packages(),
    include_package_data=True,
    license='Apache 2.0',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
    author='Javier Cordero Martinez',
    author_email='j@jcmz.me'
)



# coding=utf-8

from django.db import models


class TestModel(models.Model):
    charfield = models.CharField(max_length=32, blank=True)
    jsonfield = models.JSONField(default=dict)

    def __str__(self):
        return str(self.pk)
